﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using Wisej.Ext.ClientFileSystem;
using Wisej.Web;
namespace Wisej.ClientFileSystem
{
    public class TreeViewHelper
    {
        public static async void GetFileSystemDirectoryHierarchy(Directory dir, TreeView tree)
        {
            // Initialize a stack
            Stack<Directory> stack = new Stack<Directory>(100);

            //Tree root
            TreeNode root = new TreeNode(dir.Name) { Tag = dir };

            //Get the highest level of the hierarchy
            Directory[] subDirectories = await dir.GetDirectoriesAsync("");

            foreach (Directory subDir in subDirectories)
            {
                stack.Push(subDir);
                TreeNode subDirNode = new TreeNode(subDir.Name) { Tag = subDir };
                root.Nodes.Add(subDirNode);
            }

            //Get the rest of the hierarchy
            while (stack.Count > 0)
            {
                Directory currentDir = stack.Pop();
                Directory[] subDirs = await currentDir.GetDirectoriesAsync("");

                TreeNode node = GetNode(currentDir.Name, root);
                if (node != null)
                {
                    foreach (Directory d in subDirs)
                    {
                        stack.Push(d);
                        node.Nodes.Add(new TreeNode(d.Name) { Tag = d });
                    }
                }
            }
            tree.Nodes.Clear();
            tree.Nodes.Add(root);
        }





        public static async Task<short> GetFileSystemHierarchy(Directory dir,TreeView tree)
        {
            // Initialize a stack
            Stack<Directory> stack = new Stack<Directory>(100);

            //Tree root
            TreeNode root = new TreeNode(dir.Name) { Tag = dir };

            //Get the highest level of the hierarchy
            Directory[] subDirectories = await dir.GetDirectoriesAsync("");
            File[] files = await dir.GetFilesAsync("");
            foreach (File file in files)
            {
                TreeNode fileNode = new TreeNode(file.Name) { Tag = file };
                root.Nodes.Add(fileNode);
            }

            foreach (Directory subDir in subDirectories)
            {
                stack.Push(subDir);
                TreeNode subDirNode = new TreeNode(subDir.Name) { Tag = subDir };
                root.Nodes.Add(subDirNode);
            }

            //Get the rest of the hierarchy
            while (stack.Count > 0)
            {
                Directory currentDir = stack.Pop();
                Directory[] subDirs = await currentDir.GetDirectoriesAsync("");

                TreeNode node = GetNode(currentDir.Name, root);
                if (node != null)
                {
                    foreach (Directory d in subDirs)
                    {
                        stack.Push(d);
                        node.Nodes.Add(new TreeNode(d.Name) { Tag = d });
                    }
                    File[] subFiles = await currentDir.GetFilesAsync("");
                    foreach (File file in subFiles)
                    {
                        node.Nodes.Add(new TreeNode(file.Name) { Tag = file });
                    }
                }
            }
            tree.Nodes.Clear();
            tree.Nodes.Add(root);

            return 1;
        }

        /// <summary>
        /// Check if a branch or node exists within another node
        /// </summary>
        /// <param name="name">Represents the branch's name</param>
        /// <param name="node">Represents the <see cref="Wisej.Web.TreeNode"/> object</param>
        /// <returns>True if the branch exists; otherwise false</returns>
        private static bool NodeExists(string name, TreeNode node)
        {
            foreach (var tc in Traverse(node.Nodes))
            {
                if (tc.Text == name)
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Returns a branch or node from a <see cref="TreeNode"/> object
        /// </summary>
        /// <param name="name">Represents the branch or node name</param>
        /// <param name="node">Represents the <see cref="Wisej.Web.TreeNode"/> object</param>
        /// <returns>Returns the requested node if found; otherwise returns null</returns>
        public static TreeNode GetNode(string name, TreeNode node)
        {
            foreach (var tc in Traverse(node.Nodes))
            {
                if (tc.Text == name)
                {
                    return tc;
                }
            }
            return null;
        }

        /// <summary>
        /// Node traversing
        /// </summary>
        static IEnumerable<TreeNode> Traverse(TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                yield return node;

                foreach (var child in Traverse(node.Nodes))
                    yield return child;
            }
        }
    }
}